"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const react_dom_1 = require("react-dom");
const use_ssr_1 = __importDefault(require("use-ssr"));
const errorMessage1 = 'You must either bind to an element or pass an event to openPortal(e).';
function usePortal(_a = {}) {
    var { closeOnOutsideClick = true, closeOnEsc = true, bindTo, // attach the portal to this node in the DOM
    isOpen: defaultIsOpen = false, onOpen, onClose } = _a, eventHandlers = __rest(_a, ["closeOnOutsideClick", "closeOnEsc", "bindTo", "isOpen", "onOpen", "onClose"]);
    const { isServer, isBrowser } = use_ssr_1.default();
    const [isOpen, makeOpen] = react_1.useState(defaultIsOpen);
    // we use this ref because `isOpen` is stale for handleOutsideMouseClick
    const open = react_1.useRef(isOpen);
    const setOpen = react_1.useCallback((v) => {
        // workaround to not have stale `isOpen` in the handleOutsideMouseClick
        open.current = v;
        makeOpen(v);
    }, []);
    const targetEl = react_1.useRef(); // this is the element you are clicking/hovering/whatever, to trigger opening the portal
    const portal = react_1.useRef(isBrowser ? document.createElement('div') : null);
    react_1.useEffect(() => {
        if (isBrowser && !portal.current)
            portal.current = document.createElement('div');
    }, [isBrowser, portal]);
    const elToMountTo = react_1.useMemo(() => {
        if (isServer)
            return;
        return (bindTo && react_dom_1.findDOMNode(bindTo)) || document.body;
    }, [isServer, bindTo]);
    const handleEvent = react_1.useCallback((func, event) => {
        if (!func || isServer)
            return;
        if (event && event.currentTarget && event.currentTarget !== document)
            targetEl.current = event.currentTarget;
        // i.e. onClick, etc. inside usePortal({ onClick({ portal, targetEl }) {} })
        func({ portal, targetEl: targetEl.current, event });
    }, [portal, targetEl]);
    // this should handle all eventHandlers like onClick, onMouseOver, etc. passed into the config
    const customEventHandlers = Object
        .entries(eventHandlers)
        .reduce((acc, [handlerName, eventHandler]) => {
        acc[handlerName] = (event) => handleEvent(eventHandler, event);
        return acc;
    }, {});
    const openPortal = react_1.useCallback((event) => {
        if (isServer)
            return;
        // for some reason, when we don't have the event argument there
        // is a weird race condition, would like to see if we can remove
        // setTimeout, but for now this works
        if (event == null && targetEl.current == null) {
            setTimeout(() => setOpen(true), 0);
            throw Error(errorMessage1);
        }
        if (event && event.nativeEvent)
            event.nativeEvent.stopImmediatePropagation();
        if (event)
            targetEl.current = event.currentTarget;
        if (!targetEl.current)
            throw Error(errorMessage1);
        if (onOpen)
            handleEvent(onOpen, event);
        setOpen(true);
    }, [isServer, portal, setOpen, handleEvent, targetEl, onOpen]);
    const closePortal = react_1.useCallback((event) => {
        if (isServer)
            return;
        if (onClose)
            handleEvent(onClose, event);
        if (open.current)
            setOpen(false);
    }, [isServer, handleEvent, onClose, setOpen]);
    const togglePortal = react_1.useCallback((e) => open.current ? closePortal(e) : openPortal(e), [closePortal, openPortal]);
    const handleKeydown = react_1.useCallback(e => {
        var ESC = 27;
        if (e.keyCode === ESC && closeOnEsc)
            closePortal(e);
    }, [closeOnEsc, closePortal]);
    const handleOutsideMouseClick = react_1.useCallback(e => {
        if (isServer)
            return;
        if (!(portal.current instanceof HTMLElement))
            return;
        if (portal.current.contains(e.target) || e.button !== 0 || !open.current || targetEl.current.contains(e.target))
            return;
        if (closeOnOutsideClick)
            closePortal(e);
    }, [isServer, closePortal, closeOnOutsideClick, portal]);
    // used to remove the event listeners on unmount
    const eventListeners = react_1.useRef({});
    react_1.useEffect(() => {
        if (isServer)
            return;
        if (!(elToMountTo instanceof HTMLElement) || !(portal.current instanceof HTMLElement))
            return;
        // TODO: eventually will need to figure out a better solution for this.
        // Surely we can find a way to map onScroll/onWheel -> scroll/wheel better,
        // but for all other event handlers. For now this works.
        const eventHandlerMap = {
            onScroll: 'scroll',
            onWheel: 'wheel',
        };
        const node = portal.current;
        elToMountTo.appendChild(portal.current);
        // handles all special case handlers. Currently only onScroll and onWheel
        Object.entries(eventHandlerMap).forEach(([handlerName /* onScroll */, eventListenerName /* scroll */]) => {
            if (!eventHandlers[handlerName])
                return;
            eventListeners.current[handlerName] = (e) => handleEvent(eventHandlers[handlerName], e);
            document.addEventListener(eventListenerName, eventListeners.current[handlerName]);
        });
        document.addEventListener('keydown', handleKeydown);
        document.addEventListener('mousedown', handleOutsideMouseClick);
        return () => {
            // handles all special case handlers. Currently only onScroll and onWheel
            Object.entries(eventHandlerMap).forEach(([handlerName, eventListenerName]) => {
                if (!eventHandlers[handlerName])
                    return;
                document.removeEventListener(eventListenerName, eventListeners.current[handlerName]);
                delete eventListeners.current[handlerName];
            });
            document.removeEventListener('keydown', handleKeydown);
            document.removeEventListener('mousedown', handleOutsideMouseClick);
            elToMountTo.removeChild(node);
        };
    }, [isServer, handleOutsideMouseClick, handleKeydown, elToMountTo, portal]);
    const Portal = react_1.useCallback(({ children }) => {
        if (portal.current != null)
            return react_dom_1.createPortal(children, portal.current);
        return null;
    }, [portal]);
    return Object.assign([openPortal, closePortal, open.current, Portal, togglePortal], Object.assign(Object.assign({ isOpen: open.current, openPortal, ref: targetEl, closePortal,
        togglePortal,
        Portal }, customEventHandlers), { bind: Object.assign({ ref: targetEl }, customEventHandlers) }));
}
exports.default = usePortal;
//# sourceMappingURL=usePortal.js.map